/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.db4ounit.common.cs;

import java.io.IOException;

import com.db4o.*;
import com.db4o.config.Configuration;
import com.db4o.foundation.io.*;

import db4ounit.*;


public class CallConstructorsConfigTestCase implements TestCase {
	
	private final int PORT = 0xdb40;
	
	public static final class Item {
	}
	
	public void test() throws IOException {
		final Configuration config = Db4o.newConfiguration();
		config.callConstructors(true);
		config.exceptionsOnNotStorable(true);
		
		final ObjectServer server = Db4o.openServer(config, databaseFile(), PORT);
		try {
			server.grantAccess("db4o", "db4o");
			
			withClient(new ClientBlock() {
				public void run(ObjectContainer client) {	
					client.set(new Item());
				}
			});
			
			withClient(new ClientBlock() {
				public void run(ObjectContainer client) {	
					Assert.areEqual(1, client.query(Item.class).size());
				}
			});
			
		} finally {
			server.close();
			File4.delete(databaseFile());
		}
	}

	private String databaseFile() {
		return Path4.combine(Path4.getTempPath(), "cc.db4o");
	}
	
	public interface ClientBlock {
		void run(ObjectContainer client);
	}

	private void withClient(ClientBlock block) throws IOException {
		final ObjectContainer client = Db4o.openClient("localhost", PORT, "db4o", "db4o");
		try {
			block.run(client);			
		} finally {
			client.close();
		}
	}

}
